@@ -1,4 +1,5 @@
 use assert_cmd::cargo::CommandCargoExt;
+use predicates::boolean::PredicateBooleanExt;
 use predicates::{prelude::predicate, str::PredicateStrExt};
 use serial_test::serial;
 use std::path::Path;
@@ -34,12 +35,17 @@ const EXAMPLES_DIR: &str = "tests/examples";
 fn bat_raw_command_with_config() -> Command {
     let mut cmd = Command::cargo_bin("bat").unwrap();
     cmd.current_dir("tests/examples");
-    cmd.env_remove("PAGER");
-    cmd.env_remove("BAT_PAGER");
+    cmd.env_remove("BAT_CACHE_PATH");
+    cmd.env_remove("BAT_CONFIG_DIR");
     cmd.env_remove("BAT_CONFIG_PATH");
+    cmd.env_remove("BAT_OPTS");
+    cmd.env_remove("BAT_PAGER");
     cmd.env_remove("BAT_STYLE");
-    cmd.env_remove("BAT_THEME");
     cmd.env_remove("BAT_TABS");
+    cmd.env_remove("BAT_THEME");
+    cmd.env_remove("COLORTERM");
+    cmd.env_remove("NO_COLOR");
+    cmd.env_remove("PAGER");
     cmd
 }
 
@@ -700,6 +706,16 @@ fn pager_failed_to_parse() {
         .stderr(predicate::str::contains("Could not parse pager command"));
 }
 
+#[test]
+fn diagnostic_sanity_check() {
+    bat()
+        .arg("--diagnostic")
+        .assert()
+        .success()
+        .stdout(predicate::str::contains("BAT_PAGER="))
+        .stderr("");
+}
+
 #[test]
 fn config_location_test() {
     bat_with_config()
@@ -737,6 +753,16 @@ fn config_location_when_generating() {
     assert!(tmp_config_path.exists());
 }
 
+#[test]
+fn config_location_from_bat_config_dir_variable() {
+    bat_with_config()
+        .env("BAT_CONFIG_DIR", "conf/")
+        .arg("--config-file")
+        .assert()
+        .success()
+        .stdout(predicate::str::is_match("conf/config\n").unwrap());
+}
+
 #[test]
 fn config_read_arguments_from_file() {
     bat_with_config()
@@ -795,6 +821,17 @@ fn does_not_print_unwanted_file_named_cache() {
     bat_with_config().arg("cach").assert().failure();
 }
 
+#[test]
+fn accepts_no_custom_assets_arg() {
+    // Just make sure --no-custom-assets is considered a valid arg
+    // Don't bother to actually verify that it works
+    bat()
+        .arg("--no-custom-assets")
+        .arg("test.txt")
+        .assert()
+        .success();
+}
+
 #[test]
 fn unicode_wrap() {
     bat_with_config()
@@ -882,7 +919,7 @@ fn empty_file_leads_to_empty_output_with_rule_enabled() {
 }
 
 #[test]
-fn filename_basic() {
+fn header_basic() {
     bat()
         .arg("test.txt")
         .arg("--decorations=always")
@@ -896,7 +933,21 @@ fn filename_basic() {
 }
 
 #[test]
-fn filename_binary() {
+fn header_full_basic() {
+    bat()
+        .arg("test.txt")
+        .arg("--decorations=always")
+        .arg("--style=header-filename,header-filesize")
+        .arg("-r=0:0")
+        .arg("--file-name=foo")
+        .assert()
+        .success()
+        .stdout("File: foo\nSize: 12 B\n")
+        .stderr("");
+}
+
+#[test]
+fn header_binary() {
     bat()
         .arg("test.binary")
         .arg("--decorations=always")
@@ -909,6 +960,20 @@ fn filename_binary() {
         .stderr("");
 }
 
+#[test]
+fn header_full_binary() {
+    bat()
+        .arg("test.binary")
+        .arg("--decorations=always")
+        .arg("--style=header-filename,header-filesize")
+        .arg("-r=0:0")
+        .arg("--file-name=foo")
+        .assert()
+        .success()
+        .stdout("File: foo   <BINARY>\nSize: 4 B\n")
+        .stderr("");
+}
+
 #[test]
 fn filename_stdin() {
     bat()
@@ -979,6 +1044,18 @@ fn header_padding() {
         .stderr("");
 }
 
+#[test]
+fn header_full_padding() {
+    bat()
+        .arg("--decorations=always")
+        .arg("--style=header-filename,header-filesize")
+        .arg("test.txt")
+        .arg("single-line.txt")
+        .assert()
+        .stdout("File: test.txt\nSize: 12 B\nhello world\n\nFile: single-line.txt\nSize: 11 B\nSingle Line\n")
+        .stderr("");
+}
+
 #[test]
 fn header_padding_rule() {
     bat()
@@ -999,6 +1076,28 @@ Single Line
         .stderr("");
 }
 
+#[test]
+fn header_full_padding_rule() {
+    bat()
+        .arg("--decorations=always")
+        .arg("--style=header-filename,header-filesize,rule")
+        .arg("--terminal-width=80")
+        .arg("test.txt")
+        .arg("single-line.txt")
+        .assert()
+        .stdout(
+            "File: test.txt
+Size: 12 B
+hello world
+────────────────────────────────────────────────────────────────────────────────
+File: single-line.txt
+Size: 11 B
+Single Line
+",
+        )
+        .stderr("");
+}
+
 #[test]
 fn grid_overrides_rule() {
     bat()
@@ -1018,7 +1117,9 @@ Single Line
 ────────────────────────────────────────────────────────────────────────────────
 ",
         )
-        .stderr("\x1b[33m[bat warning]\x1b[0m: Style 'rule' is a subset of style 'grid', 'rule' will not be visible.\n");
+        .stderr(
+            "\x1b[33m[bat warning]\x1b[0m: Style 'rule' is a subset of style 'grid', 'rule' will not be visible.\n",
+        );
 }
 
 #[cfg(target_os = "linux")]
@@ -1091,6 +1192,21 @@ fn do_not_detect_different_syntax_for_stdin_and_files() {
     );
 }
 
+#[test]
+fn no_first_line_fallback_when_mapping_to_invalid_syntax() {
+    let file = "regression_tests/first_line_fallback.invalid-syntax";
+
+    bat()
+        .arg("--color=always")
+        .arg("--map-syntax=*.invalid-syntax:InvalidSyntax")
+        .arg(&format!("--file-name={}", file))
+        .arg("--style=plain")
+        .arg(file)
+        .assert()
+        .failure()
+        .stderr(predicate::str::contains("unknown syntax: 'InvalidSyntax'"));
+}
+
 #[test]
 fn show_all_mode() {
     bat()
@@ -1101,6 +1217,42 @@ fn show_all_mode() {
         .stderr("");
 }
 
+#[test]
+fn no_paging_arg() {
+    bat()
+        .arg("--no-paging")
+        .arg("--color=never")
+        .arg("--decorations=never")
+        .arg("single-line.txt")
+        .assert()
+        .success()
+        .stdout("Single Line");
+}
+
+#[test]
+fn no_paging_short_arg() {
+    bat()
+        .arg("-P")
+        .arg("--color=never")
+        .arg("--decorations=never")
+        .arg("single-line.txt")
+        .assert()
+        .success()
+        .stdout("Single Line");
+}
+
+#[test]
+fn no_pager_arg() {
+    bat()
+        .arg("--no-pager")
+        .arg("--color=never")
+        .arg("--decorations=never")
+        .arg("single-line.txt")
+        .assert()
+        .success()
+        .stdout("Single Line");
+}
+
 #[test]
 fn plain_mode_does_not_add_nonexisting_newline() {
     bat()
@@ -1131,6 +1283,7 @@ fn grid_for_file_without_newline() {
             "\
 ───────┬────────────────────────────────────────────────────────────────────────
        │ File: single-line.txt
+       │ Size: 11 B
 ───────┼────────────────────────────────────────────────────────────────────────
    1   │ Single Line
 ───────┴────────────────────────────────────────────────────────────────────────
@@ -1138,3 +1291,147 @@ fn grid_for_file_without_newline() {
         )
         .stderr("");
 }
+
+// For ANSI theme, use underscore as a highlighter
+#[test]
+fn ansi_highlight_underline() {
+    bat()
+        .arg("--paging=never")
+        .arg("--color=never")
+        .arg("--terminal-width=80")
+        .arg("--wrap=never")
+        .arg("--decorations=always")
+        .arg("--theme=ansi")
+        .arg("--style=plain")
+        .arg("--highlight-line=1")
+        .write_stdin("Ansi Underscore Test\nAnother Line")
+        .assert()
+        .success()
+        .stdout("\x1B[4mAnsi Underscore Test\n\x1B[24mAnother Line")
+        .stderr("");
+}
+
+// Ensure that ANSI passthrough is emitted properly for both wrapping and non-wrapping printer.
+#[test]
+fn ansi_passthrough_emit() {
+    for wrapping in &["never", "character"] {
+        bat()
+            .arg("--paging=never")
+            .arg("--color=never")
+            .arg("--terminal-width=80")
+            .arg(format!("--wrap={}", wrapping))
+            .arg("--decorations=always")
+            .arg("--style=plain")
+            .write_stdin("\x1B[33mColor\nColor \x1B[m\nPlain\n")
+            .assert()
+            .success()
+            .stdout("\x1B[33m\x1B[33mColor\n\x1B[33mColor \x1B[m\nPlain\n")
+            .stderr("");
+    }
+}
+
+#[test]
+fn ignored_suffix_arg() {
+    bat()
+        .arg("-f")
+        .arg("-p")
+        .arg("test.json~")
+        .assert()
+        .success()
+        .stdout("\u{1b}[38;5;231m{\u{1b}[0m\u{1b}[38;5;208m\"\u{1b}[0m\u{1b}[38;5;208mtest\u{1b}[0m\u{1b}[38;5;208m\"\u{1b}[0m\u{1b}[38;5;231m:\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;186m\"\u{1b}[0m\u{1b}[38;5;186mvalue\u{1b}[0m\u{1b}[38;5;186m\"\u{1b}[0m\u{1b}[38;5;231m}\u{1b}[0m")
+        .stderr("");
+
+    bat()
+        .arg("-f")
+        .arg("-p")
+        .arg("--ignored-suffix=.suffix")
+        .arg("test.json.suffix")
+        .assert()
+        .success()
+        .stdout("\u{1b}[38;5;231m{\u{1b}[0m\u{1b}[38;5;208m\"\u{1b}[0m\u{1b}[38;5;208mtest\u{1b}[0m\u{1b}[38;5;208m\"\u{1b}[0m\u{1b}[38;5;231m:\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;186m\"\u{1b}[0m\u{1b}[38;5;186mvalue\u{1b}[0m\u{1b}[38;5;186m\"\u{1b}[0m\u{1b}[38;5;231m}\u{1b}[0m")
+        .stderr("");
+
+    bat()
+        .arg("-f")
+        .arg("-p")
+        .arg("test.json.suffix")
+        .assert()
+        .success()
+        .stdout("\u{1b}[38;5;231m{\"test\": \"value\"}\u{1b}[0m")
+        .stderr("");
+}
+
+#[test]
+fn all_global_git_config_locations_syntax_mapping_work() {
+    let fake_home = Path::new(EXAMPLES_DIR).join("git").canonicalize().unwrap();
+    let expected = "\u{1b}[38;5;231m[\u{1b}[0m\u{1b}[38;5;149muser\u{1b}[0m\u{1b}[38;5;231m]\u{1b}[0m
+\u{1b}[38;5;231m    \u{1b}[0m\u{1b}[38;5;231memail\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;203m=\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;186mfoo@bar.net\u{1b}[0m
+\u{1b}[38;5;231m    \u{1b}[0m\u{1b}[38;5;231mname\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;203m=\u{1b}[0m\u{1b}[38;5;231m \u{1b}[0m\u{1b}[38;5;186mfoobar\u{1b}[0m
+";
+
+    bat()
+        .env("XDG_CONFIG_HOME", fake_home.join(".config").as_os_str())
+        .arg("-f")
+        .arg("-p")
+        .arg("git/.config/git/config")
+        .assert()
+        .success()
+        .stdout(expected)
+        .stderr("");
+
+    bat()
+        .env("HOME", fake_home.as_os_str())
+        .arg("-f")
+        .arg("-p")
+        .arg("git/.config/git/config")
+        .assert()
+        .success()
+        .stdout(expected)
+        .stderr("");
+
+    bat()
+        .env("HOME", fake_home.as_os_str())
+        .arg("-f")
+        .arg("-p")
+        .arg("git/.gitconfig")
+        .assert()
+        .success()
+        .stdout(expected)
+        .stderr("");
+}
+
+#[test]
+fn acknowledgements() {
+    bat()
+        .arg("--acknowledgements")
+        .assert()
+        .success()
+        .stdout(
+            // Just some sanity checking that avoids names of persons, except our own Keith Hall :)
+            predicate::str::contains(
+                "Copyright (c) 2018-2021 bat-developers (https://github.com/sharkdp/bat).",
+            )
+            .and(predicate::str::contains(
+                "Copyright (c) 2012-2020 The Sublime CMake authors",
+            ))
+            .and(predicate::str::contains(
+                "Copyright 2014-2015 SaltStack Team",
+            ))
+            .and(predicate::str::contains(
+                "Copyright (c) 2013-present Dracula Theme",
+            ))
+            .and(predicate::str::contains(
+                "## syntaxes/01_Packages/Rust/LICENSE.txt",
+            ))
+            .and(predicate::str::contains(
+                "## syntaxes/02_Extra/http-request-response/LICENSE",
+            ))
+            .and(predicate::str::contains(
+                "## themes/dracula-sublime/LICENSE",
+            ))
+            .and(predicate::str::contains("Copyright (c) 2017 b123400"))
+            .and(predicate::str::contains("Copyright (c) 2021 Keith Hall"))
+            .and(predicate::str::contains("Copyright 2014 Clams")),
+        )
+        .stderr("");
+}
